@@ -9,11 +9,10 @@
 
 package Data::Dumper;
 
-$VERSION = '2.121_13';
-
+$VERSION = '2.121_14'
 #$| = 1;
 
-use 5.006_001;
+;
 require Exporter;
 require overload;
 
@@ -253,6 +252,13 @@ sub Dumpperl {
   return wantarray ? @out : join('', @out);
 }
 
+# wrap string in single quotes (escaping if needed)
+sub _quote {
+    my $val = shift;
+    $val =~ s/([\\\'])/\\$1/g;
+    return  "'" . $val .  "'";
+}
+
 #
 # twist, toil and turn;
 # and recurse, of course.
@@ -274,7 +280,7 @@ sub _dump {
     # implementation.
     my $freezer = $s->{freezer};
     if ($freezer and UNIVERSAL::can($val, $freezer)) {
-      eval { $val->$freezer() };
+      eval { $val->?$freezer() };
       warn "WARNING(Freezer method call failed): $@" if $@;
     }
 
@@ -438,7 +444,7 @@ sub _dump {
     }
     
     if ($realpack) { # we have a blessed ref
-      $out .= ', \'' . $realpack . '\'' . ' )';
+      $out .= ', ' . _quote($realpack) . ' )';
       $out .= '->' . $s->{toaster} . '()'  if $s->{toaster} ne '';
       $s->{apad} = $blesspad;
     }
@@ -502,8 +508,7 @@ sub _dump {
 	$out .= qquote($val, $s->{useqq});
       }
       else {
-	$val =~ s/([\\\'])/\\$1/g;
-	$out .= '\'' . $val .  '\'';
+        $out .= _quote($val);
       }
     }
   }
