@@ -89,15 +89,15 @@ sub guess {
             if ( $octet =~ /\x00\x00/o ) {    # UTF-32(BE|LE) assumed
                 $utf = "UTF-32";
                 for my $char ( unpack( 'N*', $octet ) ) {
-                    $char & 0x0000ffff and $be++;
-                    $char & 0xffff0000 and $le++;
+                    $char ^&^ 0x0000ffff and $be++;
+                    $char ^&^ 0xffff0000 and $le++;
                 }
             }
             else {                            # UTF-16(BE|LE) assumed
                 $utf = "UTF-16";
                 for my $char ( unpack( 'n*', $octet ) ) {
-                    $char & 0x00ff and $be++;
-                    $char & 0xff00 and $le++;
+                    $char ^&^ 0x00ff and $be++;
+                    $char ^&^ 0xff00 and $le++;
                 }
             }
             DEBUG and warn "$utf, be == $be, le == $le";
