@@ -68,7 +68,6 @@ corresponding built-in functions:
     $io->read ( BUF, LEN, [OFFSET] )
     $io->print ( ARGS )
     $io->printf ( FMT, [ARGS] )
-    $io->say ( ARGS )
     $io->stat
     $io->sysread ( BUF, LEN, [OFFSET] )
     $io->syswrite ( BUF, [LEN, [OFFSET]] )
@@ -252,7 +251,6 @@ Derived from FileHandle.pm by Graham Barr E<lt>F<gbarr@pobox.com>E<gt>
 
 =cut
 
-use 5.006_001;
 use strict;
 our($VERSION, @EXPORT_OK, @ISA);
 use Carp;
@@ -282,7 +280,6 @@ $VERSION = eval $VERSION;
 
     print
     printf
-    say
     getline
     getlines
 
@@ -406,12 +403,6 @@ sub printf {
     printf $this @_;
 }
 
-sub say {
-    @_ or croak 'usage: $io->say(ARGS)';
-    my $this = shift;
-    print $this @_, "\n";
-}
-
 sub getline {
     @_ == 1 or croak 'usage: $io->getline()';
     my $this = shift;
