@@ -1,7 +1,6 @@
 
 package IO::Uncompress::Gunzip ;
 
-require 5.004 ;
 
 # for RFC1952
 
@@ -9,12 +8,12 @@ use strict ;
 use warnings;
 use bytes;
 
-use IO::Uncompress::RawInflate 2.004 ;
+use IO::Uncompress::RawInflate 2.006 ;
 
-use Compress::Raw::Zlib 2.004 qw( crc32 ) ;
-use IO::Compress::Base::Common 2.004 qw(:Status createSelfTiedObject);
-use IO::Compress::Gzip::Constants 2.004 ;
-use IO::Compress::Zlib::Extra 2.004 ;
+use Compress::Raw::Zlib 2.006 qw( crc32 ) ;
+use IO::Compress::Base::Common 2.006 qw(:Status createSelfTiedObject);
+use IO::Compress::Gzip::Constants 2.006 ;
+use IO::Compress::Zlib::Extra 2.006 ;
 
 require Exporter ;
 
@@ -28,7 +27,7 @@ Exporter::export_ok_tags('all');
 
 $GunzipError = '';
 
-$VERSION = '2.004';
+$VERSION = '2.006';
 
 sub new
 {
@@ -47,7 +46,7 @@ sub gunzip
 
 sub getExtraParams
 {
-    use IO::Compress::Base::Common  2.004 qw(:Parse);
+    use IO::Compress::Base::Common  2.006 qw(:Parse);
     return ( 'ParseExtra' => [1, 1, Parse_boolean,  0] ) ;
 }
 
@@ -165,11 +164,11 @@ sub _readGzipHeader($)
 
     # check for use of reserved bits
     return $self->HeaderError("Use of Reserved Bits in FLG field.")
-        if $flag & GZIP_FLG_RESERVED ; 
+        if $flag ^&^ GZIP_FLG_RESERVED ; 
 
     my $EXTRA ;
     my @EXTRA = () ;
-    if ($flag & GZIP_FLG_FEXTRA) {
+    if ($flag ^&^ GZIP_FLG_FEXTRA) {
         $EXTRA = "" ;
         $self->smartReadExact(\$buffer, GZIP_FEXTRA_HEADER_SIZE) 
             or return $self->TruncatedHeader("FEXTRA Length") ;
@@ -188,7 +187,7 @@ sub _readGzipHeader($)
     }
 
     my $origname ;
-    if ($flag & GZIP_FLG_FNAME) {
+    if ($flag ^&^ GZIP_FLG_FNAME) {
         $origname = "" ;
         while (1) {
             $self->smartReadExact(\$buffer, 1) 
@@ -203,7 +202,7 @@ sub _readGzipHeader($)
     }
 
     my $comment ;
-    if ($flag & GZIP_FLG_FCOMMENT) {
+    if ($flag ^&^ GZIP_FLG_FCOMMENT) {
         $comment = "";
         while (1) {
             $self->smartReadExact(\$buffer, 1) 
@@ -217,12 +216,12 @@ sub _readGzipHeader($)
             if *$self->{Strict} && $comment =~ /$GZIP_FCOMMENT_INVALID_CHAR_RE/o ;
     }
 
-    if ($flag & GZIP_FLG_FHCRC) {
+    if ($flag ^&^ GZIP_FLG_FHCRC) {
         $self->smartReadExact(\$buffer, GZIP_FHCRC_SIZE) 
             or return $self->TruncatedHeader("FHCRC");
 
         $HeaderCRC = unpack("v", $buffer) ;
-        my $crc16 = crc32($keep) & 0xFF ;
+        my $crc16 = crc32($keep) ^&^ 0xFF ;
 
         return $self->HeaderError("CRC16 mismatch.")
             if *$self->{Strict} && $crc16 != $HeaderCRC;
@@ -246,11 +245,11 @@ sub _readGzipHeader($)
 
         'MethodID'      => $cm,
         'MethodName'    => $cm == GZIP_CM_DEFLATED ? "Deflated" : "Unknown" ,
-        'TextFlag'      => $flag & GZIP_FLG_FTEXT ? 1 : 0,
-        'HeaderCRCFlag' => $flag & GZIP_FLG_FHCRC ? 1 : 0,
-        'NameFlag'      => $flag & GZIP_FLG_FNAME ? 1 : 0,
-        'CommentFlag'   => $flag & GZIP_FLG_FCOMMENT ? 1 : 0,
-        'ExtraFlag'     => $flag & GZIP_FLG_FEXTRA ? 1 : 0,
+        'TextFlag'      => $flag ^&^ GZIP_FLG_FTEXT ? 1 : 0,
+        'HeaderCRCFlag' => $flag ^&^ GZIP_FLG_FHCRC ? 1 : 0,
+        'NameFlag'      => $flag ^&^ GZIP_FLG_FNAME ? 1 : 0,
+        'CommentFlag'   => $flag ^&^ GZIP_FLG_FCOMMENT ? 1 : 0,
+        'ExtraFlag'     => $flag ^&^ GZIP_FLG_FEXTRA ? 1 : 0,
         'Name'          => $origname,
         'Comment'       => $comment,
         'Time'          => $mtime,
