@@ -5,15 +5,15 @@ use strict ;
 use warnings;
 use bytes;
 
-use IO::Compress::Base::Common  2.004 qw(:Status createSelfTiedObject);
-use IO::Compress::Zlib::Constants 2.004 ;
+use IO::Compress::Base::Common  2.006 qw(:Status createSelfTiedObject);
+use IO::Compress::Zlib::Constants 2.006 ;
 
-use IO::Uncompress::RawInflate  2.004 ;
+use IO::Uncompress::RawInflate  2.006 ;
 
 require Exporter ;
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $InflateError);
 
-$VERSION = '2.004';
+$VERSION = '2.006';
 $InflateError = '';
 
 @ISA    = qw( IO::Uncompress::RawInflate Exporter );
@@ -132,7 +132,7 @@ sub bits
     my $offset = shift ;
     my $mask  = shift ;
 
-    ($data >> $offset ) & $mask & 0xFF ;
+    ($data >> $offset ) ^&^ $mask ^&^ 0xFF ;
 }
 
 
