@@ -1,6 +1,5 @@
 package Safe;
 
-use 5.003_11;
 use strict;
 
 $Safe::VERSION = "2.12";
@@ -38,7 +37,55 @@ use Opcode 1.01, qw(
 
 
 my $default_root  = 0;
-my $default_share = ['*_']; #, '*main::'];
+# share *_ and functions defined in universal.c
+# Don't share stuff like *UNIVERSAL:: otherwise code from the
+# compartment can 0wn functions in UNIVERSAL
+my $default_share = [qw[
+    *_
+    &PerlIO::get_layers
+    &Regexp::DESTROY
+    &re::is_regexp
+    &re::regname
+    &re::regnames
+    &re::regnames_count
+    &Tie::Hash::NamedCapture::FETCH
+    &Tie::Hash::NamedCapture::STORE
+    &Tie::Hash::NamedCapture::DELETE
+    &Tie::Hash::NamedCapture::CLEAR
+    &Tie::Hash::NamedCapture::EXISTS
+    &Tie::Hash::NamedCapture::FIRSTKEY
+    &Tie::Hash::NamedCapture::NEXTKEY
+    &Tie::Hash::NamedCapture::SCALAR
+    &Tie::Hash::NamedCapture::flags
+    &UNIVERSAL::isa
+    &UNIVERSAL::can
+    &UNIVERSAL::DOES
+    &UNIVERSAL::VERSION
+    &utf8::is_utf8
+    &utf8::valid
+    &utf8::encode
+    &utf8::decode
+    &utf8::upgrade
+    &utf8::downgrade
+    &utf8::native_to_unicode
+    &utf8::unicode_to_native
+    &version::()
+    &version::new
+    &version::(""
+    &version::stringify
+    &version::(0+
+    &version::numify
+    &version::normal
+    &version::(cmp
+    &version::(<=>
+    &version::vcmp
+    &version::(bool
+    &version::boolean
+    &version::(nomethod
+    &version::noop
+    &version::is_alpha
+    &version::qv
+]];
 
 sub new {
     my($class, $root, $mask) = @_;
@@ -139,7 +186,7 @@ sub untrap { shift->permit(@_) }
 
 sub deny {
     my $obj = shift;
-    $obj->{Mask} |= opset(@_);
+    $obj->{Mask} ^|^= opset(@_);
 }
 sub deny_only {
     my $obj = shift;
@@ -149,7 +196,7 @@ sub deny_only {
 sub permit {
     my $obj = shift;
     # XXX needs testing
-    $obj->{Mask} &= invert_opset opset(@_);
+    $obj->{Mask} ^&^= invert_opset opset(@_);
 }
 sub permit_only {
     my $obj = shift;
@@ -183,9 +230,6 @@ sub share_from {
     my $arg;
     foreach $arg (@$vars) {
 	# catch some $safe->share($var) errors:
-	croak("'$arg' not a valid symbol table name")
-	    unless $arg =~ /^[\$\@%*&]?\w[\w:]*$/
-	    	or $arg =~ /^\$\W$/;
 	my ($var, $type);
 	$type = $1 if ($var = $arg) =~ s/^(\W)//;
 	# warn "share_from $pkg $type $var";
