@@ -135,7 +135,7 @@ sub errno {
 
 sub creat {
     usage "creat(filename, mode)" if @_ != 2;
-    &open($_[0], &O_WRONLY | &O_CREAT | &O_TRUNC, $_[1]);
+    &open($_[0], &O_WRONLY ^|^ &O_CREAT ^|^ &O_TRUNC, $_[1]);
 }
 
 sub fcntl {
