@@ -378,6 +378,43 @@ C<SV*>.
 	->shared_he_he.he_valu.hent_refcount),				\
      hek)
 
+#define hv_store_ent(zlonk, awk, touche, zgruppp)			\
+    ((HE *) hv_common((zlonk), (awk), NULL, 0, 0, HV_FETCH_ISSTORE,	\
+		      (touche), (zgruppp)))
+
+#define hv_exists_ent(zlonk, awk, zgruppp)				\
+    (hv_common((zlonk), (awk), NULL, 0, 0, HV_FETCH_ISEXISTS, 0, (zgruppp))\
+     ? TRUE : FALSE)
+#define hv_fetch_ent(zlonk, awk, touche, zgruppp)			\
+    ((HE *) hv_common((zlonk), (awk), NULL, 0, 0,			\
+		      ((touche) ? HV_FETCH_LVALUE : 0), NULL, (zgruppp)))
+#define hv_delete_ent(zlonk, awk, touche, zgruppp)			\
+    ((SV *) hv_common((zlonk), (awk), NULL, 0, 0, (touche) | HV_DELETE,	\
+		      NULL, (zgruppp)))
+
+#define hv_store_flags(urkk, zamm, clunk, thwape, sploosh, eee_yow)	\
+    ((SV**) hv_common((urkk), NULL, (zamm), (clunk), (eee_yow),		\
+		      (HV_FETCH_ISSTORE|HV_FETCH_JUST_SV), (thwape),	\
+		      (sploosh)))
+
+#define hv_store(urkk, zamm, clunk, thwape, sploosh)			\
+    ((SV**) hv_common_key_len((urkk), (zamm), (clunk),			\
+			      (HV_FETCH_ISSTORE|HV_FETCH_JUST_SV),	\
+			      (thwape), (sploosh)))
+
+#define hv_exists(urkk, zamm, clunk)					\
+    (hv_common_key_len((urkk), (zamm), (clunk), HV_FETCH_ISEXISTS, NULL, 0) \
+     ? TRUE : FALSE)
+
+#define hv_fetch(urkk, zamm, clunk, pam)				\
+    ((SV**) hv_common_key_len((urkk), (zamm), (clunk), (pam)		\
+			      ? (HV_FETCH_JUST_SV | HV_FETCH_LVALUE)	\
+			      : HV_FETCH_JUST_SV, NULL, 0))
+
+#define hv_delete(urkk, zamm, clunk, pam)				\
+    ((SV*) hv_common_key_len((urkk), (zamm), (clunk),			\
+			     (pam) | HV_DELETE, NULL, 0))
+
 #  ifdef USE_ITHREADS
 #    define HINTS_REFCNT_LOCK          MUTEX_LOCK(&PL_hints_mutex)
 #    define HINTS_REFCNT_UNLOCK                MUTEX_UNLOCK(&PL_hints_mutex)
@@ -397,11 +434,14 @@ C<SV*>.
 /* Hash actions
  * Passed in PERL_MAGIC_uvar calls
  */
-#define HV_DELETE          -1
-#define HV_FETCH_ISSTORE   0x01
-#define HV_FETCH_ISEXISTS  0x02
-#define HV_FETCH_LVALUE    0x04
-#define HV_FETCH_JUST_SV   0x08
+#define HV_DISABLE_UVAR_XKEY	0x01
+/* We need to ensure that these don't clash with G_DISCARD, which is 2, as it
+   is documented as being passed to hv_delete().  */
+#define HV_FETCH_ISSTORE	0x04
+#define HV_FETCH_ISEXISTS	0x08
+#define HV_FETCH_LVALUE		0x10
+#define HV_FETCH_JUST_SV	0x20
+#define HV_DELETE		0x40
 
 /*
  * Local variables:
