@@ -1,8 +1,8 @@
 package CPAN::HandleConfig;
 use strict;
-use vars qw(%can %keys $VERSION);
+use vars qw(%can %keys $loading $VERSION);
 
-$VERSION = sprintf "%.6f", substr(q$Rev: 1566 $,4)/1000000 + 5.4;
+$VERSION = sprintf "%.6f", substr(q$Rev: 2212 $,4)/1000000 + 5.4;
 
 %can = (
         commit   => "Commit changes to disk",
@@ -48,6 +48,7 @@ $VERSION = sprintf "%.6f", substr(q$Rev: 1566 $,4)/1000000 + 5.4;
      "index_expire",
      "inhibit_startup_message",
      "keep_source_where",
+     "load_module_verbosity",
      "lynx",
      "make",
      "make_arg",
@@ -65,15 +66,18 @@ $VERSION = sprintf "%.6f", substr(q$Rev: 1566 $,4)/1000000 + 5.4;
      "password",
      "patch",
      "prefer_installer",
-     "prerequisites_policy",
      "prefs_dir",
+     "prerequisites_policy",
      "proxy_pass",
      "proxy_user",
      "randomize_urllist",
      "scan_cache",
      "shell",
+     "show_unparsable_versions",
      "show_upload_date",
+     "show_zero_versions",
      "tar",
+     "tar_verbosity",
      "term_is_latin",
      "term_ornaments",
      "test_report",
@@ -83,6 +87,7 @@ $VERSION = sprintf "%.6f", substr(q$Rev: 1566 $,4)/1000000 + 5.4;
      "username",
      "wait_list",
      "wget",
+     "yaml_load_code",
      "yaml_module",
     );
 
@@ -120,8 +125,8 @@ sub edit {
     $o = shift @args;
     $DB::single = 1;
     if($can{$o}) {
-	$self->$o(args => \@args); # o conf init => sub init => sub load
-	return 1;
+        $self->?$o(args => \@args); # o conf init => sub init => sub load
+        return 1;
     } else {
         CPAN->debug("o[$o]") if $CPAN::DEBUG;
         unless (exists $keys{$o}) {
@@ -132,40 +137,40 @@ sub edit {
 
         # one day I used randomize_urllist for a boolean, so we must
         # list them explicitly --ak
-	if (0) {
+        if (0) {
         } elsif ($o =~ /^(wait_list|urllist|dontload_list)$/) {
 
             #
             # ARRAYS
             #
 
-	    $func = shift @args;
-	    $func ||= "";
+            $func = shift @args;
+            $func ||= "";
             CPAN->debug("func[$func]args[@args]") if $CPAN::DEBUG;
-	    # Let's avoid eval, it's easier to comprehend without.
-	    if ($func eq "push") {
-		push @{$CPAN::Config->{$o}}, @args;
+            # Let's avoid eval, it's easier to comprehend without.
+            if ($func eq "push") {
+                push @{$CPAN::Config->{$o}}, @args;
                 $changed = 1;
-	    } elsif ($func eq "pop") {
-		pop @{$CPAN::Config->{$o}};
+            } elsif ($func eq "pop") {
+                pop @{$CPAN::Config->{$o}};
                 $changed = 1;
-	    } elsif ($func eq "shift") {
-		shift @{$CPAN::Config->{$o}};
+            } elsif ($func eq "shift") {
+                shift @{$CPAN::Config->{$o}};
                 $changed = 1;
-	    } elsif ($func eq "unshift") {
-		unshift @{$CPAN::Config->{$o}}, @args;
+            } elsif ($func eq "unshift") {
+                unshift @{$CPAN::Config->{$o}}, @args;
                 $changed = 1;
-	    } elsif ($func eq "splice") {
+            } elsif ($func eq "splice") {
                 my $offset = shift @args || 0;
                 my $length = shift @args || 0;
-		splice @{$CPAN::Config->{$o}}, $offset, $length, @args; # may warn
+                splice @{$CPAN::Config->{$o}}, $offset, $length, @args; # may warn
                 $changed = 1;
-	    } elsif ($func) {
-		$CPAN::Config->{$o} = [$func, @args];
+            } elsif ($func) {
+                $CPAN::Config->{$o} = [$func, @args];
                 $changed = 1;
-	    } else {
+            } else {
                 $self->prettyprint($o);
-	    }
+            }
             if ($changed) {
                 if ($o eq "urllist") {
                     # reset the cached values
@@ -183,7 +188,7 @@ sub edit {
             # HASHES
             #
 
-            if (@args==1 && $args[0] eq ""){
+            if (@args==1 && $args[0] eq "") {
                 @args = ();
             } elsif (@args % 2) {
                 push @args, "";
@@ -196,14 +201,14 @@ sub edit {
             # SCALARS
             #
 
-            if (defined $args[0]){
+            if (defined $args[0]) {
                 $CPAN::CONFIG_DIRTY = 1;
                 $CPAN::Config->{$o} = $args[0];
                 $changed = 1;
             }
-	    $self->prettyprint($o)
+            $self->prettyprint($o)
                 if exists $keys{$o} or defined $CPAN::Config->{$o};
-	}
+        }
         if ($changed) {
             if ($CPAN::Config->{auto_commit}) {
                 $self->commit;
@@ -217,33 +222,35 @@ sub edit {
 }
 
 sub prettyprint {
-  my($self,$k) = @_;
-  my $v = $CPAN::Config->{$k};
-  if (ref $v) {
-    my(@report);
-    if (ref $v eq "ARRAY") {
-      @report = map {"\t$_ \[$v->[$_]]\n"} 0..$#$v;
+    my($self,$k) = @_;
+    my $v = $CPAN::Config->{$k};
+    if (ref $v) {
+        my(@report);
+        if (ref $v eq "ARRAY") {
+            @report = map {"\t$_ \[$v->[$_]]\n"} 0..$#$v;
+        } else {
+            @report = map
+                {
+                    sprintf "\t%-18s => %s\n",
+                               "[$_]",
+                                        defined $v->{$_} ? "[$v->{$_}]" : "undef"
+                } keys %$v;
+        }
+        $CPAN::Frontend->myprint(
+                                 join(
+                                      "",
+                                      sprintf(
+                                              "    %-18s\n",
+                                              $k
+                                             ),
+                                      @report
+                                     )
+                                );
+    } elsif (defined $v) {
+        $CPAN::Frontend->myprint(sprintf "    %-18s [%s]\n", $k, $v);
     } else {
-      @report = map { sprintf("\t%-18s => %s\n",
-                              map { "[$_]" } $_,
-                              defined $v->{$_} ? $v->{$_} : "UNDEFINED"
-                             )} keys %$v;
+        $CPAN::Frontend->myprint(sprintf "    %-18s undef\n", $k);
     }
-    $CPAN::Frontend->myprint(
-                             join(
-                                  "",
-                                  sprintf(
-                                          "    %-18s\n",
-                                          $k
-                                         ),
-                                  @report
-                                 )
-                            );
-  } elsif (defined $v) {
-    $CPAN::Frontend->myprint(sprintf "    %-18s [%s]\n", $k, $v);
-  } else {
-    $CPAN::Frontend->myprint(sprintf "    %-18s [%s]\n", $k, "UNDEFINED");
-  }
 }
 
 sub commit {
@@ -264,10 +271,10 @@ sub commit {
         $configpm = $args[0];
       }
     }
-    unless (defined $configpm){
-	$configpm ||= $INC{"CPAN/MyConfig.pm"};
-	$configpm ||= $INC{"CPAN/Config.pm"};
-	$configpm || Carp::confess(q{
+    unless (defined $configpm) {
+        $configpm ||= $INC{"CPAN/MyConfig.pm"};
+        $configpm ||= $INC{"CPAN/Config.pm"};
+        $configpm || Carp::confess(q{
 CPAN::Config::commit called without an argument.
 Please specify a filename where to save the configuration or try
 "o conf init" to have an interactive course through configing.
@@ -275,10 +282,10 @@ Please specify a filename where to save the configuration or try
     }
     my($mode);
     if (-f $configpm) {
-	$mode = (stat $configpm)[2];
-	if ($mode && ! -w _) {
-	    Carp::confess("$configpm is not writable");
-	}
+        $mode = (stat $configpm)[2];
+        if ($mode && ! -w _) {
+            Carp::confess("$configpm is not writable");
+        }
     }
 
     my $msg;
@@ -298,15 +305,15 @@ EOF
     $fh->print(qq[$msg\$CPAN::Config = \{\n]);
     foreach (sort keys %$CPAN::Config) {
         unless (exists $keys{$_}) {
-            $CPAN::Frontend->mywarn("Dropping unknown config variable '$_'\n");
-            delete $CPAN::Config->{$_};
+            # do not drop them: forward compatibility!
+            $CPAN::Frontend->mywarn("Unknown config variable '$_'\n");
             next;
         }
-	$fh->print(
-		   "  '$_' => ",
-		   $self->neatvalue($CPAN::Config->{$_}),
-		   ",\n"
-		  );
+        $fh->print(
+            "  '$_' => ",
+            $self->neatvalue($CPAN::Config->{$_}),
+            ",\n"
+        );
     }
 
     $fh->print("};\n1;\n__END__\n");
@@ -327,7 +334,7 @@ sub neatvalue {
     my($self, $v) = @_;
     return "undef" unless defined $v;
     my($t) = ref $v;
-    unless ($t){
+    unless ($t) {
         $v =~ s/\\/\\\\/g;
         return "q[$v]";
     }
@@ -343,7 +350,7 @@ sub neatvalue {
     }
     return "$v" unless $t eq 'HASH';
     my(@m, $key, $val);
-    while (($key,$val) = each %$v){
+    while (($key,$val) = each %$v) {
         last unless defined $key; # cautious programming in case (undef,undef) is true
         push(@m,"q[$key]=>".$self->neatvalue($val)) ;
     }
@@ -408,7 +415,7 @@ else: quote it with the correct quote type for the box we're on
     my ($quotes,$use_quote)
         = $^O eq 'MSWin32'
             ? ('"', '"')
-                : (q<"'>, "'")
+                : (q{"'}, "'")
                     ;
 
     sub safe_quote {
@@ -428,12 +435,8 @@ else: quote it with the correct quote type for the box we're on
 
 sub init {
     my($self,@args) = @_;
-    undef $CPAN::Config->{'inhibit_startup_message'}; # lazy trick to
-                                                      # have the least
-                                                      # important
-                                                      # variable
-                                                      # undefined
-    $self->load(@args);
+    CPAN->debug("self[$self]args[".join(",",@args)."]");
+    $self->load(doit => 1, @args);
     1;
 }
 
@@ -441,7 +444,7 @@ sub init {
 # maintainability.  RMB
 #
 sub _configpmtest {
-    my($configpmdir, $configpmtest) = @_; 
+    my($configpmdir, $configpmtest) = @_;
     if (-w $configpmtest) {
         return $configpmtest;
     } elsif (-w $configpmdir) {
@@ -450,20 +453,20 @@ sub _configpmtest {
         unlink $configpm_bak if -f $configpm_bak;
         if( -f $configpmtest ) {
             if( rename $configpmtest, $configpm_bak ) {
-				$CPAN::Frontend->mywarn(<<END);
+                $CPAN::Frontend->mywarn(<<END);
 Old configuration file $configpmtest
     moved to $configpm_bak
 END
-	    }
-	}
-	my $fh = FileHandle->new;
-	if ($fh->open(">$configpmtest")) {
-	    $fh->print("1;\n");
-	    return $configpmtest;
-	} else {
-	    # Should never happen
-	    Carp::confess("Cannot open >$configpmtest");
-	}
+            }
+        }
+        my $fh = FileHandle->new;
+        if ($fh->open(">$configpmtest")) {
+            $fh->print("1;\n");
+            return $configpmtest;
+        } else {
+            # Should never happen
+            Carp::confess("Cannot open >$configpmtest");
+        }
     } else { return }
 }
 
@@ -490,7 +493,11 @@ sub home () {
     my $home;
     if ($CPAN::META->has_usable("File::HomeDir")) {
         $home = File::HomeDir->my_data;
-    } else {
+        unless (defined $home) {
+            $home = File::HomeDir->my_home
+        }
+    }
+    unless (defined $home) {
         $home = $ENV{HOME};
     }
     $home;
@@ -498,39 +505,42 @@ sub home () {
 
 sub load {
     my($self, %args) = @_;
-	$CPAN::Be_Silent++ if $args{be_silent};
+    $CPAN::Be_Silent++ if $args{be_silent};
+    my $doit;
+    $doit = delete $args{doit};
 
-    my(@miss);
     use Carp;
     require_myconfig_or_config;
-    return unless @miss = $self->missing_config_data;
+    my @miss = $self->missing_config_data;
+    return unless $doit || @miss;
+    return if $loading;
+    $loading++;
 
     require CPAN::FirstTime;
-    my($configpm,$fh,$redo,$theycalled);
+    my($configpm,$fh,$redo);
     $redo ||= "";
-    $theycalled++ if @miss==1 && $miss[0] eq 'inhibit_startup_message';
     if (defined $INC{"CPAN/Config.pm"} && -w $INC{"CPAN/Config.pm"}) {
-	$configpm = $INC{"CPAN/Config.pm"};
-	$redo++;
+        $configpm = $INC{"CPAN/Config.pm"};
+        $redo++;
     } elsif (defined $INC{"CPAN/MyConfig.pm"} && -w $INC{"CPAN/MyConfig.pm"}) {
-	$configpm = $INC{"CPAN/MyConfig.pm"};
-	$redo++;
+        $configpm = $INC{"CPAN/MyConfig.pm"};
+        $redo++;
     } else {
-	my($path_to_cpan) = File::Basename::dirname($INC{"CPAN.pm"});
-	my($configpmdir) = File::Spec->catdir($path_to_cpan,"CPAN");
-	my($configpmtest) = File::Spec->catfile($configpmdir,"Config.pm");
+        my($path_to_cpan) = File::Basename::dirname($INC{"CPAN.pm"});
+        my($configpmdir) = File::Spec->catdir($path_to_cpan,"CPAN");
+        my($configpmtest) = File::Spec->catfile($configpmdir,"Config.pm");
         my $inc_key;
-	if (-d $configpmdir or File::Path::mkpath($configpmdir)) {
-	    $configpm = _configpmtest($configpmdir,$configpmtest);
+        if (-d $configpmdir or File::Path::mkpath($configpmdir)) {
+            $configpm = _configpmtest($configpmdir,$configpmtest);
             $inc_key = "CPAN/Config.pm";
-	}
-	unless ($configpm) {
-	    $configpmdir = File::Spec->catdir(home,".cpan","CPAN");
-	    File::Path::mkpath($configpmdir);
-	    $configpmtest = File::Spec->catfile($configpmdir,"MyConfig.pm");
-	    $configpm = _configpmtest($configpmdir,$configpmtest);
+        }
+        unless ($configpm) {
+            $configpmdir = File::Spec->catdir(home,".cpan","CPAN");
+            File::Path::mkpath($configpmdir);
+            $configpmtest = File::Spec->catfile($configpmdir,"MyConfig.pm");
+            $configpm = _configpmtest($configpmdir,$configpmtest);
             $inc_key = "CPAN/MyConfig.pm";
-	}
+        }
         if ($configpm) {
           $INC{$inc_key} = $configpm;
         } else {
@@ -541,22 +551,17 @@ sub load {
 
     }
     local($") = ", ";
-    if ($redo && ! $theycalled){
+    if ($redo && !$doit) {
         $CPAN::Frontend->myprint(<<END);
 Sorry, we have to rerun the configuration dialog for CPAN.pm due to
-the following indispensable but missing parameters:
+some missing parameters...
 
-@miss
 END
         $args{args} = \@miss;
     }
-    if (0) {
-        # where do we need this?
-        $CPAN::Frontend->myprint(qq{
-$configpm initialized.
-});
-    }
     CPAN::FirstTime::init($configpm, %args);
+    $loading--;
+    return;
 }
 
 
@@ -573,7 +578,7 @@ sub missing_config_data {
          #"gzip",
          "http_proxy",
          "index_expire",
-         "inhibit_startup_message",
+         #"inhibit_startup_message",
          "keep_source_where",
          #"make",
          "make_arg",
@@ -592,7 +597,7 @@ sub missing_config_data {
          "urllist",
         ) {
         next unless exists $keys{$_};
-	push @miss, $_ unless defined $CPAN::Config->{$_};
+        push @miss, $_ unless defined $CPAN::Config->{$_};
     }
     return @miss;
 }
@@ -622,17 +627,17 @@ sub cpl {
     CPAN->debug("word[$word] line[$line] pos[$pos]") if $CPAN::DEBUG;
     my(@words) = split " ", substr($line,0,$pos+1);
     if (
-	defined($words[2])
-	and
+        defined($words[2])
+        and
         $words[2] =~ /list$/
         and
-	(
-	 @words == 3
-	 ||
-	 @words == 4 && length($word)
-	)
+        (
+        @words == 3
+        ||
+        @words == 4 && length($word)
+        )
        ) {
-	return grep /^\Q$word\E/, qw(splice shift unshift pop push);
+        return grep /^\Q$word\E/, qw(splice shift unshift pop push);
     } elsif (defined($words[2])
              and
              $words[2] eq "init"
@@ -642,9 +647,9 @@ sub cpl {
              ||
              @words >= 4 && length($word)
             )) {
-	return sort grep /^\Q$word\E/, keys %keys;
+        return sort grep /^\Q$word\E/, keys %keys;
     } elsif (@words >= 4) {
-	return ();
+        return ();
     }
     my %seen;
     my(@o_conf) =  sort grep { !$seen{$_}++ }
@@ -685,14 +690,15 @@ sub prefs_lookup {
 
     use strict;
     use vars qw($AUTOLOAD $VERSION);
-    $VERSION = sprintf "%.2f", substr(q$Rev: 1566 $,4)/100;
+    $VERSION = sprintf "%.2f", substr(q$Rev: 2212 $,4)/100;
 
     # formerly CPAN::HandleConfig was known as CPAN::Config
     sub AUTOLOAD {
+        my $class = shift; # e.g. in dh-make-perl: CPAN::Config
         my($l) = $AUTOLOAD;
         $CPAN::Frontend->mywarn("Dispatching deprecated method '$l' to CPAN::HandleConfig\n");
         $l =~ s/.*:://;
-        CPAN::HandleConfig->$l(@_);
+        CPAN::HandleConfig->?$l(@_);
     }
 }
 
