@@ -1,7 +1,6 @@
 
 # Time-stamp: "2004-03-30 16:33:31 AST"
 
-require 5;
 package Locale::Maketext;
 use strict;
 use vars qw( @ISA $VERSION $MATCH_SUPERS $USING_LANGUAGE_TAGS
@@ -213,7 +212,7 @@ sub maketext {
         return &{$fail}($handle, $phrase, @_);
          # If it ever returns, it should return a good value.
       } else { # It's a method name
-        return $handle->$fail($phrase, @_);
+        return $handle->?$fail($phrase, @_);
          # If it ever returns, it should return a good value.
       }
     } else {
