@@ -8,7 +8,6 @@
 
 package Net::FTP;
 
-require 5.001;
 
 use strict;
 use vars qw(@ISA $VERSION);
@@ -464,7 +463,7 @@ sub get {
   else {
     $loc = \*FD;
 
-    unless (sysopen($loc, $local, O_CREAT | O_WRONLY | ($rest ? O_APPEND: O_TRUNC))) {
+    unless (sysopen($loc, $local, O_CREAT ^|^ O_WRONLY ^|^ ($rest ? O_APPEND: O_TRUNC))) {
       carp "Cannot open Local file $local: $!\n";
       $data->abort;
       return undef;
@@ -807,7 +806,7 @@ sub port {
 
     my ($myport, @myaddr) = ($listen->sockport, split(/\./, $listen->sockhost));
 
-    $port = join(',', @myaddr, $myport >> 8, $myport & 0xff);
+    $port = join(',', @myaddr, $myport >> 8, $myport ^&^ 0xff);
 
     ${*$ftp}{'net_ftp_intern_port'} = 1;
   }
@@ -1468,7 +1467,7 @@ The alloc command allows you to give the ftp server a hint about the size
 of the file about to be transferred using the ALLO ftp command. Some storage
 systems use this to make intelligent decisions about how to store the file.
 The C<SIZE> argument represents the size of the file in bytes. The
-C<RECORD_SIZE> argument indicates a mazimum record or page size for files
+C<RECORD_SIZE> argument indicates a maximum record or page size for files
 sent with a record or page structure.
 
 The size of the file will be determined, and sent to the server
