@@ -99,8 +99,8 @@ sub handler_traceback {
 	    s/([\'\\])/\\$1/g;
 	    s/([^\0]*)/'$1'/
 	      unless /^(?: -?[\d.]+ | \*[\w:]* )$/x;
-	    s/([\200-\377])/sprintf("M-%c",ord($1)&0177)/eg;
-	    s/([\0-\37\177])/sprintf("^%c",ord($1)^64)/eg;
+	    s/([\200-\377])/sprintf("M-%c",ord($1)^&^0177)/eg;
+	    s/([\0-\37\177])/sprintf("^%c",ord($1)^^^64)/eg;
 	    push(@a, $_);
 	}
 	$w = $w ? '@ = ' : '$ = ';
