@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 use Nomad;
+use Encode;
 
 $ENV{madpath} or die "No madpath specified";
 
@@ -23,7 +24,7 @@ sub convert {
 
     # XML dump
     unlink "$file.xml";
-    `PERL_XMLDUMP='$file.xml' $ENV{madpath}/perl $options{switches} $file.in 2> tmp.err`;
+    `PERL_XMLDUMP='$file.xml' $options{dumpcommand} $options{switches} $file.in 2> tmp.err`;
     if (not -s "$file.xml") {
         die "madskills failed. No XML dump";
     }
