@@ -1,5 +1,4 @@
 #!/usr/bin/perl -w
-require 5.003;	# keep this compatible, an old perl is all we may have before
                 # we build the new one
 
 # The idea is to move the regen_headers target out of the Makefile so that
@@ -18,7 +17,6 @@ safer_unlink ("warnings.h", "lib/warnings.pm");
 
 my %gen = (
 	   'autodoc.pl'  => [qw[pod/perlapi.pod pod/perlintern.pod]],
-	   'bytecode.pl' => [qw[ext/B/B/Asmdata.pm]],
 	   'embed.pl'    => [qw[proto.h embed.h embedvar.h global.sym
 				perlapi.h perlapi.c]],
 	   'keywords.pl' => [qw[keywords.h]],
@@ -45,7 +43,7 @@ sub do_cksum {
     return %cksum;
 }
 
-foreach my $pl (qw (keywords.pl opcode.pl embed.pl bytecode.pl
+foreach my $pl (qw (keywords.pl opcode.pl embed.pl
 		    regcomp.pl warnings.pl autodoc.pl reentr.pl)) {
   print "$^X $pl\n";
   my %cksum0;
