@@ -14,7 +14,7 @@ BEGIN {
     # use Test::NoWarnings, if available
     my $extra = 0 ;
     $extra = 1
-        if eval { require Test::NoWarnings ;  Test::NoWarnings->import(); 1 };
+        if eval { require Test::NoWarnings ;  'Test::NoWarnings'->import(); 1 };
 
     plan tests => 970 + $extra ;
 
@@ -353,7 +353,7 @@ sub run
 
                     ok ! -e $out_file, "  Output file does not exist";
                     writeFile($out_file, $already);
-                    my $of = IO::File->new( ">>$out_file") ;
+                    my $of = 'IO::File'->new( ">>$out_file") ;
                     ok $of, "  Created output filehandle" ;
 
                     ok &$Func(\$buffer, $of, AutoClose => 1, Append => $append), '  Compressed ok' ;
@@ -391,7 +391,7 @@ sub run
 
                     ok ! -e $out_file, "  Output file does not exist";
                     writeFile($out_file, $already);
-                    my $out = IO::File->new( ">>$out_file") ;
+                    my $out = 'IO::File'->new( ">>$out_file") ;
 
                     ok &$Func($in_file, $out, AutoClose => 1, Append => $append), '  Compressed ok' ;
 
@@ -423,7 +423,7 @@ sub run
 
                     my $lex = LexFile->new(my $in_file, my $out_file) ;
                     writeFile($in_file, $buffer);
-                    my $in = IO::File->new( "<$in_file") ;
+                    my $in = 'IO::File'->new( "<$in_file") ;
 
                     ok ! -e $out_file, "  Output file does not exist";
                     writeFile($out_file, $already);
@@ -443,11 +443,11 @@ sub run
 
                     my $lex = LexFile->new(my $in_file, my $out_file) ;
                     writeFile($in_file, $buffer);
-                    my $in = IO::File->new( "<$in_file") ;
+                    my $in = 'IO::File'->new( "<$in_file") ;
 
                     ok ! -e $out_file, "  Output file does not exist";
                     writeFile($out_file, $already);
-                    my $out = IO::File->new( ">>$out_file") ;
+                    my $out = 'IO::File'->new( ">>$out_file") ;
 
                     ok &$Func($in, $out, AutoClose => 1, Append => $append), '  Compressed ok' ;
 
@@ -463,7 +463,7 @@ sub run
 
                     my $lex = LexFile->new(my $in_file, my $out_file) ;
                     writeFile($in_file, $buffer);
-                    my $in = IO::File->new( "<$in_file") ;
+                    my $in = 'IO::File'->new( "<$in_file") ;
 
                     my $out = $already ;
 
@@ -515,7 +515,7 @@ sub run
 
         writeFile($file1, "data1");
         writeFile($file2, "data2");
-        my $of = IO::File->new( "<$file1") ;
+        my $of = 'IO::File'->new( "<$file1") ;
         ok $of, "  Created output filehandle" ;
 
         #my @input = (   undef, "", $file2, \undef, \'', \"abcde", $of) ;
@@ -590,7 +590,7 @@ sub run
 
                 my $lex = LexFile->new(my $file3) ;
 
-                my $fh3 = IO::File->new( ">$file3");
+                my $fh3 = 'IO::File'->new( ">$file3");
 
                 # rewind the filehandle
                 $of->open("<$file1") ;
@@ -644,7 +644,7 @@ sub run
             {
                 writeFile($name, $compressed);
 
-                $input = IO::File->new( "<$name") ;
+                $input = 'IO::File'->new( "<$name") ;
             }
 
             my $trailing;
@@ -950,7 +950,7 @@ sub run
 
                     my $filename = "abcde";
                     my $lex = LexFile->new($filename) ;
-                    my $fh = IO::File->new( ">$filename");
+                    my $fh = 'IO::File'->new( ">$filename");
                     
                     ok &$Func("<$tmpDir1/a*.tmp>" => $fh, 
                               MultiStream => $ms, AutoClose => 1), '  Compressed ok' 
@@ -1046,11 +1046,11 @@ sub run
                 my $of ;
                 if ($append) {
                     writeFile($out_file, $incumbent) ;
-                    $of = IO::File->new( "+< $out_file") ;
+                    $of = 'IO::File'->new( "+< $out_file") ;
                 }
                 else {
                     ok ! -e $out_file, "  Output file does not exist" ;
-                    $of = IO::File->new( "> $out_file") ;
+                    $of = 'IO::File'->new( "> $out_file") ;
                 }
                 isa_ok $of, 'IO::File', '  $of' ;
 
@@ -1090,11 +1090,11 @@ sub run
                 my $out ;
                 if ($append) {
                     writeFile($out_file, $incumbent) ;
-                    $out = IO::File->new( "+< $out_file") ;
+                    $out = 'IO::File'->new( "+< $out_file") ;
                 }
                 else {
                     ok ! -e $out_file, "  Output file does not exist" ;
-                    $out = IO::File->new( "> $out_file") ;
+                    $out = 'IO::File'->new( "> $out_file") ;
                 }
                 isa_ok $out, 'IO::File', '  $out' ;
 
@@ -1134,7 +1134,7 @@ sub run
                   { ok ! -e $out_file, "  Output file does not exist" }
 
                 writeFile($in_file, $comp);
-                my $in = IO::File->new( "<$in_file") ;
+                my $in = 'IO::File'->new( "<$in_file") ;
 
                 ok &$Func($in, $out_file, Append => $append, @opts), '  Uncompressed ok' ;
 
@@ -1152,16 +1152,16 @@ sub run
                 my $out ;
                 if ($append) {
                     writeFile($out_file, $incumbent) ;
-                    $out = IO::File->new( "+< $out_file") ;
+                    $out = 'IO::File'->new( "+< $out_file") ;
                 }
                 else {
                     ok ! -e $out_file, "  Output file does not exist" ;
-                    $out = IO::File->new( "> $out_file") ;
+                    $out = 'IO::File'->new( "> $out_file") ;
                 }
                 isa_ok $out, 'IO::File', '  $out' ;
 
                 writeFile($in_file, $comp);
-                my $in = IO::File->new( "<$in_file") ;
+                my $in = 'IO::File'->new( "<$in_file") ;
 
                 ok &$Func($in, $out, Append => $append, AutoClose => 1, @opts), '  Uncompressed ok' ;
 
@@ -1177,7 +1177,7 @@ sub run
 
                 my $lex = LexFile->new(my $in_file) ;
                 writeFile($in_file, $comp);
-                my $in = IO::File->new( "<$in_file") ;
+                my $in = 'IO::File'->new( "<$in_file") ;
 
                 my $output ;
                 $output = $incumbent if $append ;
@@ -1221,7 +1221,7 @@ sub run
             my $appended = 'appended';
             my $len_appended = length $appended;
             writeFile($in_file, $comp . $appended . $comp . $appended) ;
-            my $in = IO::File->new( "<$in_file") ;
+            my $in = 'IO::File'->new( "<$in_file") ;
 
             ok &$Func($in, \$out, Transparent => 0, InputLength => length $comp, @opts), '  Uncompressed ok' ;
 
@@ -1299,7 +1299,7 @@ sub run
         writeFile($file1, compressBuffer(getTopFuncName($UncompressClass),"data1"));
         writeFile($file2, compressBuffer(getTopFuncName($UncompressClass),"data2"));
 
-        my $of = IO::File->new( "<$file1") ;
+        my $of = 'IO::File'->new( "<$file1") ;
         ok $of, "  Created output filehandle" ;
 
         #my @input    = ($file2, \$undef, \$null, \$comp, $of) ;
@@ -1333,7 +1333,7 @@ sub run
             title "$TopType - From ArrayRef to Filehandle" ;
 
             my $lex = LexFile->new( my $output);
-            my $fh = IO::File->new( ">$output") ;
+            my $fh = 'IO::File'->new( ">$output") ;
             $of->open("<$file1") ;
 
             ok &$Func(\@input, $fh, AutoClose => 0, @opts), '  UnCompressed ok' ;
@@ -1445,7 +1445,7 @@ sub run
 
             my $output = 'abc' ;
             my $lex = LexFile->new( $output) ;
-            my $fh = IO::File->new( ">$output") ;
+            my $fh = 'IO::File'->new( ">$output") ;
             ok &$Func("<$tmpDir1/a*.tmp>" => $fh, AutoClose => 1, @opts), '  UnCompressed ok' 
                 or diag $$Error ;
 
@@ -1518,9 +1518,9 @@ sub run
         {
             my ($send, $get) = @$data ;
 
-            my $fh1 = IO::File->new( "< $file1") ;
-            my $fh2 = IO::File->new( "< $file2") ;
-            my $fh3 = IO::File->new( "< $file3") ;
+            my $fh1 = 'IO::File'->new( "< $file1") ;
+            my $fh2 = 'IO::File'->new( "< $file2") ;
+            my $fh3 = 'IO::File'->new( "< $file3") ;
 
             title "$send";
             my($copy);
