@@ -530,8 +530,8 @@ sub runperl {
 	local $ENV{'DCL$PATH'} = $1 if $is_vms && ($ENV{'DCL$PATH'} =~ /(.*)/s);
 	$ENV{PATH} =~ /(.*)/s;
 	local $ENV{PATH} =
-	    join $sep, grep { $_ ne "" and $_ ne "." and
-		($is_mswin or $is_vms or !(stat && (stat '_')[2]&0022)) }
+	    join $sep, grep { $_ ne "" and $_ ne "." and -d $_ and
+		($is_mswin or $is_vms or !(stat && (stat '_')[2]^&^0022)) }
 		    split quotemeta ($sep), $1;
 
 	$runperl =~ /(.*)/s;
